""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.0
Author  : Stefano Covino
Date    : 16/12/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : DaoPSFPhot

Remarks :

History : (24/02/2016) First version.
        : (13/07/2016) Case of single object.
        : (16/12/2016) FWHM in PSF computation.
"""


import SRPGW as GW
import numpy
from PythonPhot import aper
from PythonPhot import getpsf
from PythonPhot import pkfit



def DaoPSFPhot(x,y,px,py,data,rds=(5,10,15),fwhm=5,zp=30.0,cent=False,bck=True):
    mag,magerr,flux,fluxerr,sky,skyerr,badflag,outstr = aper.aper(data,px,py,phpadu=GW.VSTgain,apr=rds[0],zeropoint=zp,skyrad=[rds[1],rds[2]],verbose=False)
    gauss,psf,psfmag = getpsf.getpsf(data,px,py,mag,sky,ronois=GW.VSTron,phpadu=GW.VSTgain,idpsf=numpy.arange(len(px)),psfrad=rds[2],fitrad=fwhm,psfname='/dev/null')
    pk = pkfit.pkfit_class(data,gauss,psf,ronois=GW.VSTron,phpadu=GW.VSTgain)
    #
    mag,magerr,flux,fluxerr,sky,skyerr,badflag,outstr = aper.aper(data,x,y,phpadu=GW.VSTgain,apr=rds[0],zeropoint=zp,skyrad=[rds[1],rds[2]],verbose=False)
    #
    pflux = []
    pfluxerr = []
    if len(x) < 2:
        x = [x,]
        y = [y,]
        sky = [sky,]
    for x,y,s in zip(x,y,sky):
        errmag,chi,sharp,niter,scale = pk.pkfit(1,x,y,s,radius=rds[0],recenter=cent)
        f = scale*10**(0.4*(zp-psfmag))
        ef = errmag*10**(0.4*(zp-psfmag))
        pflux.append(float(f))
        pfluxerr.append(float(ef))
    #
    return flux.flatten(), fluxerr.flatten(), pflux, pfluxerr
    



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 06/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetPics

Remarks :

History : (06/11/2015) First version.
"""

import os
import astropy.io.fits as aif
import aplpy
from SRPGW.GetFits import GetFits


def GetPicsA (ra, dec, picid, fname, size=15., osize=2.5):
    ffn = GetFits(ra,dec,"_temp",fname,0.5)
    #
    f = aplpy.FITSFigure(ffn)
    os.remove(ffn)
    f.show_grayscale(invert=True,pmax=99.)
    f.recenter(ra,dec,radius=size/3600.)
    f.show_circles(ra, dec, osize/3600.,edgecolor='blue')
    f.frame.set_linewidth(2)
    f.frame.set_color('black')
    f.add_grid()
    f.grid.show()
    picfname = picid+'_'+os.path.splitext(os.path.split(fname)[-1])[0]+'.png'
    f.save(picfname)
    return picfname
    


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.1
Author  : Stefano Covino
Date    : 08/02/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (03/11/2015) First version.
        : (20/01/2016) Rome MySQL server query.
        : (08/02/2016) Rome server catalogue name.
"""

from astropy.table import Table
from astropy.coordinates import SkyCoord
from astropy.coordinates import search_around_sky
from astropy import units as u
from astroquery.vizier import Vizier
import warnings
import mysql.connector
import SRPGW as GW



def GetIGAIA(rad,decd,rds,myquery=False):
    if myquery == GW.IGAIABO or myquery == GW.IGAIARM:
        gaia = 'No'
        if myquery == GW.IGAIABO:
            cnx = mysql.connector.connect(user=GW.IGAIAUser, password=GW.IGAIAPwd, host=GW.IGAIAHost, database=GW.IGAIADB)
        else:
            cnx = mysql.connector.connect(user=GW.IGAIAUserRM, password=GW.IGAIAPwdRM, host=GW.IGAIAHostRM, database=GW.IGAIADBRM)
        #
        cursor = cnx.cursor()
        if myquery == GW.IGAIABO:
            query = "SELECT * FROM %s WHERE dif_circle(%s,%s,%s)" % (GW.IGAIACatBO,rad,decd,rds/60.0)
        else:
            query = "SELECT * FROM %s WHERE dif_circle(%s,%s,%s)" % (GW.IGAIACatRM,rad,decd,rds/60.0)
        #
        cursor.execute(query)
        res = []
        for i in cursor:
            res.append(i)
        if len(res) > 0:
            for l in range(len(cursor.column_names)):
                if cursor.column_names[l] == GW.IGAIACol:
                    gaia = 'IGAIA'+str(res[0][l])
                    break
        cursor.close()
        cnx.close()

    #
    else:
        crd = SkyCoord(ra=rad*u.degree, dec=decd*u.degree)
        #
        warnings.filterwarnings('ignore', category=UserWarning, append=True)
        try:
            restab = Vizier.query_region(crd, radius=rds*u.arcsec, catalog=["GAIA"])
        except:
            restab = []
        warnings.resetwarnings()
        if len(restab) == 0:
            gaia = "No"
        else:
            try:
                gaia = 'IGAIA'+str(restab[0][GW.IGAIACol][0])
            except KeyError:
                gaia = 'IGAIA'+str(restab[0][GW.IGAIAColb][0])
            #
    return gaia



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 06/03/2019
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : 

Remarks :

History : (06/03/2019) First version.
"""

import numpy
from SRP.SRPTime.UT2MJD import UT2MJD
import SRPGW as GW



def ReadDate(res,header):
    if header == GW.DATEHEAD:
        return res
    elif header == 'JD':
        return res - GW.JD2MJD
    elif header == 'DATE-OBS':
        if res.find('T') >= 0:
            gyyg,gttg = res.split('T')
            gyg,gmog,gdg = gyyg.split('-')
            ghg,gmig,gsg = gttg.split(':')
            return UT2MJD(int(gyg),int(gmog),int(gdg),int(ghg),int(gmig),float(gsg))
        else:
            return -99.
    else:
        return -99.
#


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 25/02/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetApPhot

Remarks :

History : (25/02/2016) First version.

"""


import SRPGW as GW
import numpy
from PythonPhot import aper


def DaoPhot(x,y,data,rds=(5,10,15),backgr=True):
    if backgr:
        mag,magerr,flux,fluxerr,sky,skyerr,badflag,outstr = aper.aper(data,x,y,phpadu=GW.VSTgain,apr=rds[0],zeropoint=30,skyrad=[rds[1],rds[2]],readnoise=GW.VSTron)
    else:
        mag,magerr,flux,fluxerr,sky,skyerr,badflag,outstr = aper.aper(data,x,y,phpadu=GW.VSTgain,apr=rds[0],zeropoint=30,skyrad=[rds[1],rds[2]],setskyval=0.0,readnoise=GW.VSTron)
    #
    return flux.flatten(),fluxerr.flatten()
    





""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 28/10/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (28/10/2015) First version.
"""


def GetCommandStr (inputstr, collist, root):
    cmdstr = []
    for en in inputstr.split():
        if en in collist:
            cmdstr.append(root+"['"+en+"'] ")
        elif en[0] == ':':
            cmdstr.append(root+"['"+en[1:]+"'] ")
        else:
            cmdstr.append(en+" ")
    return "".join(cmdstr)



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.0
Author  : Stefano Covino
Date    : 07/12/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (17/02/2016) First version.
        : (14/06/2016) Column added even if no matches.
        : (30/08/2016) Bug correction.
        : (07/12/2016) Better management of input coordinates and masks.
"""

from astropy.table import Table, join, unique
from astropy import units as u
import SRPGW as GW
from astroquery.xmatch import XMatch



def GetSimbadxMatch(tab, rds, cccol=(GW.RACOL,GW.DECCOL)):
    itab = Table([tab[cccol[0]],tab[cccol[1]]])
    otab = XMatch.query(cat1=itab,cat2='simbad',max_distance=rds*u.arcsec,colRA1=cccol[0],colDec1=cccol[1])
    #
    otabsel = Table([otab[cccol[0]],otab[cccol[1]],otab['main_id']])
    #
    if len(otabsel) > 0:
        newtab = join(tab, otabsel, join_type='left')
    else:
        tab[GW.SIMBCOL] = 'No'
        newtab = tab
    #
    try:
        newtab2 = unique(newtab,GW.IDCOL)
    except KeyError:
        newtab2 = newtab
    #
    if 'main_id' in newtab2.columns.keys():
        newtab2['main_id'][newtab2['main_id'].mask == True] = 'No'
        #
        if GW.SIMBCOL in newtab2.columns.keys():
            newtab2.remove_column(GW.SIMBCOL)
        newtab2.rename_column('main_id',GW.SIMBCOL)
    #
    return newtab2



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.0
Author  : Stefano Covino
Date    : 16/05/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (06/03/2016) First version.
        : (16/05/2016) Better managent of not fully standard astrometry.
"""


from SRP.SRPCatalogue.GetVizierCat import GetVizierCat
from astropy.wcs import WCS
import astropy.wcs
from SRPGW.GetHeadVal import GetHeadVal
import numpy
from astropy.table import Table
from astropy.coordinates import SkyCoord
from astropy.coordinates import search_around_sky
from astropy import units as u
import astropy.units.core
import SRPGW as GW
import astLib.astWCS as aLW




def GetSkyBotMatch(rad,decd,fname,time,rds,maxerr):
    skybotaddress = "vo.imcce.fr"
    skybotquery = "/webservices/skybot/skybotconesearch_query.php?-ep=%s&-ra=%.5f&-dec=%.5f&-rd=%.2f&-mime=text&-outout=object&-filter=%.1f&-from=SRP"
    #
    APYWCS = False
    try:
        w = WCS(fname)
        APYWCS = True
    except astropy.wcs._wcs.InvalidTransformError:
        w = aLW.WCS(fname)
        w.NUMPY_MODE = False
    #
    nx1 = GetHeadVal('NAXIS1',fname)
    nx2 = GetHeadVal('NAXIS2',fname)
    #
    if APYWCS:
        rac, decc = w.all_pix2world(nx1/2,nx2/2,0,ra_dec_order=True)
    else:
        RADEC = w.pix2wcs(nx1/2,nx2/2)
        rac = RADEC[0]
        decc = RADEC[1]
    #
    data = GetVizierCat(skybotaddress,skybotquery % (time,rac,decc,2.0,maxerr))
    #
    objl = []
    ral = []
    decl = []
    magl = []
    #
    if data != None:
        for en in data:
            il = en.split('|')
            if len(il) > 7:
                objid = il[1].strip()
                ra = float(il[2])*15.0
                dec = float(il[3])
                objvmag = float(il[5])
                #
                objl.append(objid)
                ral.append(ra)
                decl.append(dec)
                magl.append(objvmag)
    #
    answ = numpy.array(['No']*len(rad),dtype='<U25')
    if len(objl) > 0:
        t = Table([objl,ral,decl,magl],names=('Object',GW.RACOL,GW.DECCOL,'Mag'))
        #
        try:
            c1 = SkyCoord(ra=rad*u.degree, dec=decd*u.degree)
        except astropy.units.core.UnitsError:
            c1 = SkyCoord(ra=rad, dec=decd)
        #
        try:
            c2 = SkyCoord(ra=t[GW.RACOL]*u.degree, dec=t[GW.DECCOL]*u.degree)
        except astropy.units.core.UnitsError:
            c2 = SkyCoord(ra=t[GW.RACOL], dec=t[GW.DECCOL])
        #
        id1, id2, d2d, d3d = search_around_sky(c1, c2, rds*u.arcsec)
        #
        if len(id1) > 0:
            answ[id1] = t[id2]['Object']
    #
    return answ




""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 25/02/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : 

Remarks :

History : (25/02/2017) First version.
"""

from astropy.table import Table, join, unique
from astropy import units as u
import SRPGW as GW
from astroquery.xmatch import XMatch



def GetUSNOxMatch(tab, rds, cccol=(GW.RACOL,GW.DECCOL)):
    itab = Table([tab[cccol[0]],tab[cccol[1]]])
    otab = XMatch.query(cat1=itab,cat2='vizier:I/284/out',max_distance=rds*u.arcsec,colRA1=cccol[0],colDec1=cccol[1])
    #
    otabsel = Table([otab[cccol[0]],otab[cccol[1]],otab['USNO-B1.0']])
    #
    if len(otabsel) > 0:
        newtab = join(tab, otabsel, join_type='left')
    else:
        tab[GW.USNOCOL] = -99
        newtab = tab
    #
    try:
        newtab2 = unique(newtab,GW.IDCOL)
    except KeyError:
        newtab2 = newtab
    #
    if 'USNO-B1.0' in newtab2.columns.keys():
        newtab2['USNO-B1.0']._sharedmask = False
        newtab2['USNO-B1.0'][newtab2['USNO-B1.0'].mask == True] = 'No'
        #
        if GW.USNOCOL in newtab2.columns.keys():
            newtab2.remove_column(GW.USNOCOL)
        newtab2.rename_column('USNO-B1.0',GW.USNOCOL)
    #
    return newtab2



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.2.1
Author  : Stefano Covino
Date    : 03/06/2019
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : DaoPSFPhot

Remarks :

History : (16/12/2016) First version.
        : (06/12/2017) PSF from image.
        : (07/12/2017) Better coding.
        : (29/05/2019) Improvements in PSF modeling.
        : (03/06/2019) Correction for PSF aperture photomerty radius.
"""


import SRPGW as GW
import numpy as np
from photutils.psf import DAOGroup
from photutils.background import MMMBackground, MADStdBackgroundRMS
from astropy.modeling.fitting import LevMarLSQFitter
from astropy.stats import gaussian_sigma_to_fwhm
from astropy.stats import sigma_clipped_stats
from photutils.psf import BasicPSFPhotometry
from astropy.table import Table
from photutils.detection import DAOStarFinder
from photutils.psf import extract_stars
from astropy.nddata import NDData
from photutils import EPSFBuilder
from photutils import CircularAperture
from photutils import aperture_photometry


OVERSAMPLING = 4

def ApyPSFPhot(x,y,px,py,data,rds=(5,10,15),fwhm=5,zp=30.0,cent=False):
    pflux = []
    pfluxerr = []
    #
    sigma_psf = fwhm/gaussian_sigma_to_fwhm
    bkgrms = MADStdBackgroundRMS()
    daogroup = DAOGroup(3.0*sigma_psf*gaussian_sigma_to_fwhm)
    mmm_bkg = MMMBackground()
    fitter = LevMarLSQFitter()
    size = np.rint(rds[0]*15).astype(np.int)
    #
    psfpos = Table(names=['x', 'y'], data=[px,py])
    mean_val, median_val, std_val = sigma_clipped_stats(data, sigma=2.)
    datas = data - median_val
    nddata = NDData(data=datas)
    stars = extract_stars(nddata, psfpos, size=2*rds[2])
    epsf_builder = EPSFBuilder(oversampling=OVERSAMPLING, maxiters=3, progress_bar=False)
    epsf, fitted_stars = epsf_builder(stars)
    #
    # compute the correction factor
    aper = CircularAperture([(epsf.shape[0]/2.,epsf.shape[1]/2.)], r=rds[0]*OVERSAMPLING)
    phap = aperture_photometry(epsf.data, aper)
    corrfact = phap['aperture_sum']
    if not cent:
        epsf.x_0.fixed = True
        epsf.y_0.fixed = True
    #
    for o in range(len(x)):
        i = np.rint(x[o]).astype(np.int)-1
        l = np.rint(y[o]).astype(np.int)-1
        image = data[l-size:l+size,i-size:i+size]
        #
        try:
            std = bkgrms(image)
        except TypeError:
            std = 0.0
        #
        daofind = DAOStarFinder(threshold=5.*std+mmm_bkg(image), fwhm=fwhm, exclude_border=True)
        dobj = daofind.find_stars(image)
        dobj.rename_column('xcentroid', 'x_0')
        dobj.rename_column('ycentroid', 'y_0')
        #
        photometry = BasicPSFPhotometry(group_maker=daogroup,bkg_estimator=mmm_bkg,psf_model=epsf,fitter=LevMarLSQFitter(),fitshape=2*int(rds[0])+1,aperture_radius=rds[0])
        #
        if len(dobj) == 0:
            pos = Table(names=['x_0', 'y_0'], data=[[x[o]-i+size],[y[o]-l+size]])
            rt = photometry(image=image, init_guesses=pos)
        else:
            rt = photometry(image=image, init_guesses=dobj)
        #
        dist = np.sqrt((rt['x_fit'] - size)**2 + (rt['y_fit'] - size)**2)
        mdist = np.min(dist)
        flu = rt[dist == mdist]['flux_fit'][0]
        flue = rt[dist == mdist]['flux_unc'][0]
        #
        flu0 = rt[dist == mdist]['flux_0'][0]
        #
        flu = flu*corrfact/OVERSAMPLING**2
        flue = flue*corrfact/OVERSAMPLING**2
        #
        pflux.append(float(flu))
        pfluxerr.append(float(flue))
        #
    return pflux, pfluxerr, pflux, pfluxerr
    



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.3.1
Author  : Stefano Covino
Date    : 19/04/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (22/01/2016) First version.
        : (02/02/2016) Bug correction.
        : (11/02/2016) Simpler and faster.
        : (26/02/2016) Centering.
        : (16/05/2016) Better management of non-fully standard astrometry.
        : (19/04/2017) SRPFITS added.
"""


import numpy
from astropy.io.fits import getdata
from astropy.wcs import WCS
import astropy.wcs
from SRPFITS.Photometry.centerMoment import centerMoment
import astLib.astWCS as aLW




def GetPixXY(rad,decd,fname,cent=True,rds=5):
    dat = getdata(fname)
    APYWCS = False
    try:
        w = WCS(fname)
        APYWCS = True
    except astropy.wcs._wcs.InvalidTransformError:
        w = aLW.WCS(fname)
        w.NUMPY_MODE = False
    #
    if APYWCS:
        x, y = w.all_world2pix(rad,decd,0)
    else:
        XY = w.wcs2pix(rad.data, decd.data)
        x = [i[0] for i in XY]
        y = [i[1] for i in XY]
    #
    if cent:
        xx = []
        yy = []
        for i,l in zip(x,y):
            try:
                a,b = centerMoment(dat,i,l,rds)
                xx.append(a)
                yy.append(b)
            except (TypeError, ValueError):
                xx.append(i)
                yy.append(l)
    else:
        xx = x
        yy = y
    #
    return xx,yy



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.3
Author  : Stefano Covino
Date    : 23/08/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetApPhot

Remarks :

History : (11/02/2016) First version.
        : (13/02/2016) Improved error estimate.
        : (18/02/2016) Quicker photometry.
        : (23/08/2017) SRPFITS version.
"""


import SRPGW as GW
#import numpy
#from photutils import CircularAperture, CircularAnnulus, aperture_photometry
#from astropy.table import hstack
import SRPFITS.Photometry.ApyPhot as SPA



def ApyPhot(x,y,data,rds=(5,10,15),backgr=True):
    return SPA.ApyPhot(x,y,data,rds,backgr,GW.VSTgain,GW.VSTron)
    




""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.1.1
Author  : Stefano Covino
Date    : 07/12/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : DaoPSFPhot

Remarks :

History : (16/12/2016) First version.
        : (06/12/2017) PSF from image.
        : (07/12/2017) Better coding.
"""


import SRPGW as GW
import numpy as np
from photutils.psf import IntegratedGaussianPRF, DAOGroup
from photutils.background import MMMBackground, MADStdBackgroundRMS
from astropy.modeling.fitting import LevMarLSQFitter
from astropy.stats import gaussian_sigma_to_fwhm
from photutils.psf import BasicPSFPhotometry
from astropy.table import Table
from scipy.stats import norm
from photutils.psf.sandbox import DiscretePRF
from photutils.detection import DAOStarFinder


def ApyPSFPhot(x,y,px,py,data,rds=(5,10,15),fwhm=5,zp=30.0,cent=False):
    pflux = []
    pfluxerr = []
    #
    corrfact = (norm.cdf(rds[0],scale=fwhm)/norm.cdf(fwhm,scale=fwhm))**2
    #corrfact = 1.
    #
    sigma_psf = fwhm/gaussian_sigma_to_fwhm
    bkgrms = MADStdBackgroundRMS()
    daogroup = DAOGroup(3.0*sigma_psf*gaussian_sigma_to_fwhm)
    mmm_bkg = MMMBackground()
    fitter = LevMarLSQFitter()
    size = np.rint(rds[0]*15).astype(np.int)
    #
    psfpos = Table(names=['x_0', 'y_0', 'flux_0'], data=[px,py,[1.]*len(px)])
    psf_model = DiscretePRF.create_from_image(data,psfpos,2*int(rds[0])+1,mode='median')
    if not cent:
        psf_model.x_0.fixed = True
        psf_model.y_0.fixed = True
    #
    for o in range(len(x)):
        i = np.rint(x[o]).astype(np.int)-1
        l = np.rint(y[o]).astype(np.int)-1
        image = data[l-size:l+size,i-size:i+size]
        #
        try:
            std = bkgrms(image)
        except TypeError:
            std = 0.0
        #
        daofind = DAOStarFinder(threshold=3.*std+mmm_bkg(image), fwhm=fwhm, exclude_border=True)
        dobj = daofind.find_stars(image)
        dobj.rename_column('xcentroid', 'x_0')
        dobj.rename_column('ycentroid', 'y_0')
        #
        photometry = BasicPSFPhotometry(group_maker=daogroup,bkg_estimator=mmm_bkg,psf_model=psf_model,fitter=LevMarLSQFitter(),fitshape=2*int(rds[0])+1,aperture_radius=rds[0])
        #
        if len(dobj) == 0:
            pos = Table(names=['x_0', 'y_0'], data=[[x[o]-i+size],[y[o]-l+size]])
            rt = photometry(image=image, init_guesses=pos)
        else:
            rt = photometry(image=image, init_guesses=dobj)
        #
        dist = np.sqrt((rt['x_fit'] - size)**2 + (rt['y_fit'] - size)**2)
        mdist = np.min(dist)
        npix = np.pi*rds[0]**2
        cbkg = npix*std**2
        flu = rt[dist == mdist]['flux_fit'][0]
        flu = flu*corrfact
        flu0 = rt[dist == mdist]['flux_0'][0]
        flue = np.sqrt(abs(flu)*GW.VSTgain + abs(cbkg)*GW.VSTgain + npix*GW.VSTron)
            #except TypeError:
            #flu = 0.0
            #flu0 = 0.0
            #flue = 0.0
        #
        pflux.append(float(flu))
        pfluxerr.append(float(flue))
        #
    return pflux, pfluxerr, pflux, pfluxerr
    



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 12/11/2015
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : FindSubPathes

Remarks :

History : (12/11/2015) First version.
"""



def FindSubPathes (col1, col2):
    posset = set()
    for p1,p2 in zip(col1,col2):
        comfin = ""
        for c1,c2 in zip(p1[::-1],p2[::-1]):
            if c1 == c2:
                comfin = comfin + c1
            else:
                posset.add(comfin[::-1])
                break
    #
    posstr = list(posset)
    posstr.sort()
    #
    resstr = set()
    for p1,p2 in zip(col1,col2):
        resstr.add((p1[:-len(posstr[0])],p2[:-len(posstr[0])]))
    return list(resstr)


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 2.0.3
Author  : Stefano Covino
Date    : 18/05/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetPics

Remarks :

History : (06/11/2015) First version.
        : (23/01/2016) APLPY version for gravitown
        : (11/02/2016) Simpler and faster.
        : (03/03/2017) More cuts label tried.
        : (18/05/2017) Minor update.
"""

import os
import astLib.astWCS as aw
import astLib.astImages as aLaI
import astLib.astPlots as aLaP
from SRPFITS.Fits.GetData import GetData
from SRPFITS.Fits.GetHeader import GetHeader
import astropy.io.fits as aif
import aplpy
from SRPGW.GetFits import GetFits
import SRPGW as GW
import pylab
from SRP.SRPSystem.Pipe import Pipe
#import astropy.stats as AS


def GetPics (ra, dec, picid, fname, size=30., osize=5., fmode=GW.ASTLIB):
    if fmode == GW.APLPY:
        ffn = GetFits(ra,dec,"_temp",fname,0.5)
        #
        f = aplpy.FITSFigure(ffn)
        os.remove(ffn)
        #
        f.show_grayscale(invert=True,pmax=99.)
        f.recenter(ra,dec,radius=size/(2*3600.))
        f.show_circles(ra, dec, osize/(2*3600.),edgecolor='blue')
        f.frame.set_linewidth(2)
        f.frame.set_color('black')
        f.add_grid()
        f.grid.show()
        picfname = picid+'_'+os.path.splitext(os.path.split(fname)[-1])[0]+'.png'
        f.save(picfname)
    #
    elif fmode == GW.ASTLIB:
        dat = GetData(fname)[0]
        hed = GetHeader(fname)[0]
        wcs = aw.WCS(hed,mode='pyfits')
        wcs.NUMPY_MODE = False
        #
        ndata = aLaI.clipImageSectionWCS(dat,wcs,ra,dec,size/3600.0)
        #mn,md,si = AS.sigma_clipped_stats(ndata['data'])
        plfc = aLaP.ImagePlot(ndata['data'],ndata['wcs'],cutLevels=["relative",99.0],colorMapName='gist_yarg')
        #plfc = aLaP.ImagePlot(ndata['data'],ndata['wcs'],cutLevels=[mn-si,mn+si],colorMapName='gist_yarg')
        #plfc = aLaP.ImagePlot(ndata['data'],ndata['wcs'],cutLevels=["smart",99.5],colorMapName='gist_yarg')
        plfc.addPlotObjects([ra],[dec],"obj",size=osize,color='blue')
        picfname = picid+'_'+os.path.splitext(os.path.split(fname)[-1])[0]+'.png'
        plfc.save(picfname)
        #
    else:
        res = os.system("SRPGWPICSStamp -i %s -o %.6f %.6f -s %.2f -n %s " % (fname,ra,dec,size/60.,picid))
        picfname = picid+'_'+os.path.splitext(os.path.split(fname)[-1])[0]+'.png'
    #
    pylab.close()
    return picfname
    



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 21/12/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : Score

Remarks :

History : (21/12/2017) First version.
"""

import numpy as np



def CheckMagDiff (m1, em1, m2, em2, th, mode='all'):
    err = np.where( ((em1 < 90) & (em2 < 90)), np.sqrt(em1**2+em2**2),
                np.where( ((em1 > 90) & (em2 < 90)), em2,
                        np.where( ((em1 < 90) & (em2 > 90) ), em1, 99.)))
    #
    if mode == 'all':
        return np.where((m1<90) & (m2<90), abs(m1-m2) >= th+err, False)
    elif mode == 'high':
        return np.where((m1<90) & (m2<90), m1-m2 >= th+err, False)
    elif mode == 'low':
        return np.where((m1<90) & (m2<90), m1-m2 <= th+err, False)




""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.1
Author  : Stefano Covino
Date    : 05/03/2019
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   :

Remarks :

History : (20/09/2016) First version.
        : (05/03/2019) GLADE 2.
"""

from astropy.table import Table, join, unique
from astropy import units as u
import SRPGW as GW
from astroquery.xmatch import XMatch



def GetGLADExMatch(tab, rds, cccol=(GW.RACOL,GW.DECCOL)):
    itab = Table([tab[cccol[0]],tab[cccol[1]]])
    #otab = XMatch.query(cat1=itab,cat2='vizier:VII/275/glade1',max_distance=rds*u.arcsec,colRA1=cccol[0],colDec1=cccol[1])
    otab = XMatch.query(cat1=itab,cat2='vizier:VII/281/glade2',max_distance=rds*u.arcsec,colRA1=cccol[0],colDec1=cccol[1])
    #
    otabsel = Table([otab[cccol[0]],otab[cccol[1]],otab['angDist']])
    #
    if len(otabsel) > 0:
        newtab = join(tab, otabsel, join_type='left')
    else:
        tab[GW.GLADECOL] = -99
        newtab = tab
    #
    try:
        newtab2 = unique(newtab,GW.IDCOL)
    except KeyError:
        newtab2 = newtab
    #
    if 'angDist' in newtab2.columns.keys():
        newtab2['angDist'][newtab2['angDist'].mask == True] = -99
        #
        if GW.GLADECOL in newtab2.columns.keys():
            newtab2.remove_column(GW.GLADECOL)
        newtab2.rename_column('angDist',GW.GLADECOL)
    #
    return newtab2



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 21/02/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   :

Remarks :

History : (21/02/2016) First version.
"""


from SRP.SRPCatalogue.GetVizierCat import GetVizierCat




def GetSkyBot(rad,decd,rds,time,maxerr):
    skybotaddress = "vo.imcce.fr"
    skybotquery = "/webservices/skybot/skybotconesearch_query.php?-ep=%s&-ra=%.5f&-dec=%.5f&-rs=%.1f&-mime=text&-outout=object&-filter=%.1f&-from=SRP"
    #
    sbot= []
    for r,d in zip(rad,decd):
        data = GetVizierCat(skybotaddress,skybotquery % (time,r,d,rds,maxerr))
        #
        if data != None:
            il = data[0].split('|')
            if len(il) < 7:
                sbot.append('No')
            else:
                objid = il[1].strip()
                ra = float(il[2])*15.0
                dec = float(il[3])
                objclass = il[4]
                objvmag = float(il[5])
                poserr = float(il[6])
                cdist = float(il[7])
                sbot.append("%s_%.2f" % (objid, objvmag))
        else:
            sbot.append('No')
    #
    return sbot



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.3
Author  : Stefano Covino
Date    : 06/12/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : GetFits

Remarks :

History : (07/01/2016) First version.
        : (11/02/2016) Simpler and faster.
        : (18/05/2017) Minor update.
        : (06/12/2017) Workaround since with astropy.io.fits functions it does not work.
"""

import os
from astropy.io import fits
import astLib.astWCS as aw
import astLib.astImages as aLaI
from SRPFITS.Fits.GetData import GetData
from SRPFITS.Fits.GetHeader import GetHeader
#import pyfits



def GetFits (ra, dec, cutname, fname, size=1.):
    dat = GetData(fname)[0]
    #hd = pyfits.open(fname)
    #hed = hd[0].header
    hed = GetHeader(fname)[0]
    wcs = aw.WCS(hed,mode='pyfits')
    wcs.NUMPY_MODE = False
    #
    ndata = aLaI.clipImageSectionWCS(dat,wcs,ra,dec,size/60.0)
    fitsfname = cutname+'_'+os.path.splitext(os.path.split(fname)[-1])[0]+'.fits'
    aLaI.saveFITS(fitsfname,ndata['data'],ndata['wcs'])
    return fitsfname
    



""" Utility functions and classes for SRP

Context : SRP
Module  : SRPGW
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/01/2017
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : AddHeaderEntry (fitsfile, keylist, entrylist, commentlist, outfilename=None)

Remarks :

History : (05/01/2017) First version.
"""


from SRP.SRPCatalogue.GetVizierCat import GetVizierCat
from astropy.table import Table




def GetPanSTARRS(rad,decd,rds):
    skybotaddress = "archive.stsci.edu"
    skybotquery = "/panstarrs/search.php?RA=%.5f&DEC=%.5f&radius=%.2f&outputformat=CSV&action=Search"
    #
    pstars= []
    for r,d in zip(rad,decd):
        data = GetVizierCat(skybotaddress,skybotquery % (r,d,rds/60.))
        #
        if data != ['no rows found']:
            dt = Table.read(data,format='ascii.csv')
            objid = dt['objName'][1]
            pstars.append("%s" % (objid))
        else:
            pstars.append('No')
    #
    return pstars
